{ pkgs, config, lib, ... }:
{
  xsession.windowManager.i3 = {
    enable = true;
    config =
      let
        # lockColors = with config.lib.stylix.colors.withHashtag; { a = base00; b = base01; d = base00; }; # Black or White, depending on current theme
        # lockColors = with config.lib.stylix.colors.withHashtag; { a = base0A; b = base0B; d = base00; }; # Green + Yellow
        lockColors = { a = "#82a401"; b = "#466c01"; d = "#648901"; }; # Old
        lockSvg = pkgs.writeText "lock.svg" "<svg xmlns=\"http://www.w3.org/2000/svg\" viewBox=\"0 0 50 50\" height=\"50\" width=\"50\"><path fill=\"${lockColors.a}\" d=\"M0 50h50V0H0z\"/><path d=\"M0 0l50 50H25L0 25zm50 0v25L25 0z\" fill=\"${lockColors.d}\"/></svg>";
        lockPng = pkgs.runCommand "lock.png" { } "${pkgs.imagemagick}/bin/convert ${lockSvg} $out";
        locker = "exec ${ pkgs.writeShellScript "i3-locker"
        ''
          # Remove SSH and GPG keys from keystores
          ${pkgs.openssh}/bin/ssh-add -D
          echo RELOADAGENT | ${pkgs.gnupg}/bin/gpg-connect-agent
          ${pkgs.coreutils}/bin/rm -rf "/tmp/cached_pass_$UID"

          ${pkgs.lightdm}/bin/dm-tool lock
          # TODO Does that work for all DMs?
          if [ $? -ne 0 ]; then
              if [ -d ${config.xdg.cacheHome}/lockpatterns ]
              then
                  pattern=$(${pkgs.findutils} ${config.xdg.cacheHome}/lockpatterns | sort -R | head -1)
              else
                  pattern=${lockPng}
              fi
              revert() {
                  ${pkgs.xorg.xset} dpms 0 0 0
              }
              trap revert SIGHUP SIGINT SIGTERM
              ${pkgs.xorg.xset} dpms 5 5 5
              ${pkgs.i3lock}/bin/i3lock --nofork --color ${builtins.substring 1 6 lockColors.d} --image=$pattern --tiling --ignore-empty-password
              revert
          fi
        ''
        }";
        focus = "exec ${ pkgs.writeShellScript "i3-focus-window"
        ''
          WINDOW=`${pkgs.xdotool}/bin/xdotool getwindowfocus`
          eval `${pkgs.xdotool}/bin/xdotool getwindowgeometry --shell $WINDOW` # this brings in variables WIDTH and HEIGHT
          TX=`${pkgs.coreutils}/bin/expr $WIDTH / 2`
          TY=`${pkgs.coreutils}/bin/expr $HEIGHT / 2`
          ${pkgs.xdotool}/bin/xdotool mousemove -window $WINDOW $TX $TY
        ''
        }";
        mode_system = "[L] Vérouillage [E] Déconnexion [S] Veille [H] Hibernation [R] Redémarrage [P] Extinction";
        mode_resize = "Resize";
        mode_pres_main = "Presentation (main display)";
        mode_pres_sec = "Presentation (secondary display)";
        mode_screen = "Screen setup [A] Auto [L] Load [S] Save [R] Remove [D] Default";
      in
      {
        terminal = "alacritty";
        # bars = []; # FIXME lemonbar
        colors = let ignore = "#ff00ff"; in
          with config.lib.stylix.colors.withHashtag; lib.mkForce {
            focused = { border = base0B; background = base0B; text = base00; indicator = base00; childBorder = base0B; };
            focusedInactive = { border = base02; background = base02; text = base05; indicator = base02; childBorder = base02; };
            unfocused = { border = base05; background = base04; text = base00; indicator = base04; childBorder = base00; };
            urgent = { border = base0F; background = base08; text = base00; indicator = base08; childBorder = base0F; };
            placeholder = { border = ignore; background = base00; text = base05; indicator = ignore; childBorder = base00; };
            background = base07;
            # I set the color of the active tab as the the background color of the terminal so they merge together.
          };
        focus.followMouse = false;
        keybindings =
          let
            mod = config.xsession.windowManager.i3.config.modifier;
            rofi = "exec --no-startup-id ${config.programs.rofi.package}/bin/rofi";
            pactl = "exec ${pkgs.pulseaudio}/bin/pactl"; # TODO Use NixOS package if using NixOS
            scrot = "${pkgs.scrot}/bin/scrot --exec '${pkgs.coreutils}/bin/mv $f ~/Screenshots/ && ${pkgs.optipng}/bin/optipng ~/Screenshots/$f'";
            # TODO nix path and create directory?
          in
          {
            # Compatibility layer for people coming from other backgrounds
            # "Mod1+Tab" = "${rofi} -modi window -show window"; # FIXME When no longer using a VM
            "Mod1+F2" = "${rofi} -modi drun -show drun";
            "Mod1+F4" = "kill";
            # kill focused window
            "${mod}+z" = "kill";
            button2 = "kill";
            # Rofi
            "${mod}+c" = "exec --no-startup-id ${pkgs.rofi-pass}/bin/rofi-pass --last-used";
            # TODO Try autopass.cr
            # 23.11 config.programs.rofi.pass.package
            "${mod}+i" = "exec --no-startup-id ${pkgs.rofimoji}/bin/rofimoji";
            "${mod}+plus" = "${rofi} -modi ssh -show ssh";
            "${mod}+ù" = "${rofi} -modi ssh -show ssh -ssh-command '{terminal} -e {ssh-client} {host} -t \"sudo -s -E\"'";
            # TODO In which keyboard layout?
            "${mod}+Tab" = "${rofi} -modi window -show window";
            # start program launcher
            "${mod}+d" = "${rofi} -modi run -show run";
            "${mod}+Shift+d" = "${rofi} -modi drun -show drun";
            # Start Applications
            "${mod}+Return" = "exec ${
            pkgs.writeShellScript "terminal" "${config.programs.alacritty.package}/bin/alacritty msg create-window || exec ${config.programs.alacritty.package}/bin/alacritty -e zsh"
            # -e zsh is for systems where I can't configure my user's shell
            # TODO Is a shell script even required?
          }";
            "${mod}+Shift+Return" = "exec ${config.programs.urxvt.package}/bin/urxvt";
            "${mod}+p" = "exec ${pkgs.xfce.thunar}/bin/tunar";
            "${mod}+m" = "exec ${config.programs.qutebrowser.package}/bin/qutebrowser --override-restore --backend=webengine";
            # TODO --backend not useful anymore
            # Volume control
            "XF86AudioRaiseVolume" = "${pactl} set-sink-mute @DEFAULT_SINK@ false; ${pactl} set-sink-volume @DEFAULT_SINK@ +5%";
            "XF86AudioLowerVolume" = "${pactl} set-sink-mute @DEFAULT_SINK@ false; ${pactl} set-sink-volume @DEFAULT_SINK@ -5%";
            "XF86AudioMute" = "${pactl} set-sink-mute @DEFAULT_SINK@ true";
            "${mod}+F7" = "${pactl} suspend-sink @DEFAULT_SINK@ 1; ${pactl} suspend-sink @DEFAULT_SINK@ 0"; # Re-synchronize bluetooth headset
            "${mod}+F11" = "exec ${pkgs.pavucontrol}/bin/pavucontrol";
            "${mod}+F12" = "exec ${pkgs.pavucontrol}/bin/pavucontrol";
            # TODO Find pacmixer?
            # Media control
            "XF86AudioPrev" = "exec ${pkgs.mpc-cli}/bin/mpc prev";
            "XF86AudioPlay" = "exec ${pkgs.mpc-cli}/bin/mpc toggle";
            "XF86AudioNext" = "exec ${pkgs.mpc-cli}/bin/mpc next";
            # Misc
            "${mod}+F10" = "exec ${ pkgs.writeShellScript "show-keyboard-layout"
            ''
              layout=`${pkgs.xorg.setxkbmap}/bin/setxkbmap -query | ${pkgs.gnugrep}/bin/grep ^layout: | ${pkgs.gawk}/bin/awk '{ print $2 }'`
              ${pkgs.libgnomekbd}/bin/gkbd-keyboard-display -l $layout
            ''
            }";
            # Screenshots
            "Print" = "exec ${scrot} --focused";
            "${mod}+Print" = "exec ${scrot}"; # FIXME Doesn't work at least in the VM
            "Ctrl+Print" = "exec ${pkgs.coreutils}/bin/sleep 1 && ${scrot} --select";
            # TODO Try using bindsym --release instead of sleep
            # change focus
            "${mod}+h" = "focus left; ${focus}";
            "${mod}+j" = "focus down; ${focus}";
            "${mod}+k" = "focus up; ${focus}";
            "${mod}+l" = "focus right; ${focus}";
            # move focused window
            "${mod}+Shift+h" = "move left; ${focus}";
            "${mod}+Shift+j" = "move down; ${focus}";
            "${mod}+Shift+k" = "move up; ${focus}";
            "${mod}+Shift+l" = "move right; ${focus}";
            # workspace back and forth (with/without active container)
            "${mod}+b" = "workspace back_and_forth; ${focus}";
            "${mod}+Shift+b" = "move container to workspace back_and_forth; workspace back_and_forth; ${focus}";
            # Change container layout
            "${mod}+g" = "split h; ${focus}";
            "${mod}+v" = "split v; ${focus}";
            "${mod}+f" = "fullscreen toggle; ${focus}";
            "${mod}+s" = "layout stacking; ${focus}";
            "${mod}+w" = "layout tabbed; ${focus}";
            "${mod}+e" = "layout toggle split; ${focus}";
            "${mod}+Shift+space" = "floating toggle; ${focus}";
            # Focus container
            "${mod}+space" = "focus mode_toggle; ${focus}";
            "${mod}+a" = "focus parent; ${focus}";
            "${mod}+q" = "focus child; ${focus}";
            # Switch to workspace
            "${mod}+1" = "workspace 1; ${focus}";
            "${mod}+2" = "workspace 2; ${focus}";
            "${mod}+3" = "workspace 3; ${focus}";
            "${mod}+4" = "workspace 4; ${focus}";
            "${mod}+5" = "workspace 5; ${focus}";
            "${mod}+6" = "workspace 6; ${focus}";
            "${mod}+7" = "workspace 7; ${focus}";
            "${mod}+8" = "workspace 8; ${focus}";
            "${mod}+9" = "workspace 9; ${focus}";
            "${mod}+0" = "workspace 10; ${focus}";
            #navigate workspaces next / previous
            "${mod}+Ctrl+h" = "workspace prev_on_output; ${focus}";
            "${mod}+Ctrl+l" = "workspace next_on_output; ${focus}";
            "${mod}+Ctrl+j" = "workspace prev; ${focus}";
            "${mod}+Ctrl+k" = "workspace next; ${focus}";
            # Move to workspace next / previous with focused container
            "${mod}+Ctrl+Shift+h" = "move container to workspace prev_on_output; workspace prev_on_output; ${focus}";
            "${mod}+Ctrl+Shift+l" = "move container to workspace next_on_output; workspace next_on_output; ${focus}";
            "${mod}+Ctrl+Shift+j" = "move container to workspace prev; workspace prev; ${focus}";
            "${mod}+Ctrl+Shift+k" = "move container to workspace next; workspace next; ${focus}";
            # move focused container to workspace
            "${mod}+ctrl+1" = "move container to workspace 1; ${focus}";
            "${mod}+ctrl+2" = "move container to workspace 2; ${focus}";
            "${mod}+ctrl+3" = "move container to workspace 3; ${focus}";
            "${mod}+ctrl+4" = "move container to workspace 4; ${focus}";
            "${mod}+ctrl+5" = "move container to workspace 5; ${focus}";
            "${mod}+ctrl+6" = "move container to workspace 6; ${focus}";
            "${mod}+ctrl+7" = "move container to workspace 7; ${focus}";
            "${mod}+ctrl+8" = "move container to workspace 8; ${focus}";
            "${mod}+ctrl+9" = "move container to workspace 9; ${focus}";
            "${mod}+ctrl+0" = "move container to workspace 10; ${focus}";
            # move to workspace with focused container
            "${mod}+shift+1" = "move container to workspace 1; workspace 1; ${focus}";
            "${mod}+shift+2" = "move container to workspace 2; workspace 2; ${focus}";
            "${mod}+shift+3" = "move container to workspace 3; workspace 3; ${focus}";
            "${mod}+shift+4" = "move container to workspace 4; workspace 4; ${focus}";
            "${mod}+shift+5" = "move container to workspace 5; workspace 5; ${focus}";
            "${mod}+shift+6" = "move container to workspace 6; workspace 6; ${focus}";
            "${mod}+shift+7" = "move container to workspace 7; workspace 7; ${focus}";
            "${mod}+shift+8" = "move container to workspace 8; workspace 8; ${focus}";
            "${mod}+shift+9" = "move container to workspace 9; workspace 9; ${focus}";
            "${mod}+shift+0" = "move container to workspace 10; workspace 10; ${focus}";
            # move workspaces to screen (arrow keys)
            "${mod}+ctrl+shift+Right" = "move workspace to output right; ${focus}";
            "${mod}+ctrl+shift+Left" = "move workspace to output left; ${focus}";
            "${mod}+Ctrl+Shift+Up" = "move workspace to output above; ${focus}";
            "${mod}+Ctrl+Shift+Down" = "move workspace to output below; ${focus}";
            # i3 control
            "${mod}+Shift+c" = "reload";
            "${mod}+Shift+r" = "restart";
            "${mod}+Shift+e" = "exit";
            # Modes
            "${mod}+Escape" = "mode ${mode_system}";
            "${mod}+r" = "mode ${mode_resize}";
            "${mod}+Shift+p" = "mode ${mode_pres_main}";
            "${mod}+t" = "mode ${mode_screen}";
          };
        modes = let return_bindings = {
          "Return" = "mode default";
          "Escape" = "mode default";
        }; in
          {
            "${mode_system}" = {
              "l" = "exec --no-startup-id ${locker}, mode default";
              "e" = "exit, mode default";
              "s" = "exec --no-startup-id ${locker} & ${pkgs.systemd}/bin/systemctl suspend, mode default";
              "h" = "exec --no-startup-id ${locker} & ${pkgs.systemd}/bin/systemctl hibernate, mode default";
              "r" = "exec --no-startup-id ${pkgs.systemd}/bin/systemctl reboot, mode default";
              "p" = "exec --no-startup-id ${pkgs.systemd}/bin/systemctl poweroff -i, mode default";
            } // return_bindings;
            "${mode_resize}" = {
              "h" = "resize shrink width 10 px or 10 ppt; ${focus}";
              "j" = "resize grow height 10 px or 10 ppt; ${focus}";
              "k" = "resize shrink height 10 px or 10 ppt; ${focus}";
              "l" = "resize grow width 10 px or 10 ppt; ${focus}";
            } // return_bindings;
            "${mode_pres_main}" = {
              "b" = "workspace 3, workspace 4, mode ${mode_pres_sec}";
              "q" = "mode default";
              "Return" = "mode default";
            };
            "${mode_pres_sec}" = {
              "b" = "workspace 1, workspace 2, mode ${mode_pres_main}";
              "q" = "mode default";
              "Return" = "mode default";
            };
            "${mode_screen}" =
              let
                builtin_configs = [ "off" "common" "clone-largest" "horizontal" "vertical" "horizontal-reverse" "vertical-reverse" ];
                autorandrmenu = { title, option, builtin ? false }: pkgs.writeShellScript "autorandrmenu"
                  ''
                    shopt -s nullglob globstar
                    profiles="${if builtin then lib.strings.concatLines builtin_configs else ""}$(${pkgs.autorandr}/bin/autorandr | ${pkgs.gawk}/bin/awk '{ print $1 }')"
                    profile="$(echo "$profiles" | ${config.programs.rofi.package}/bin/rofi -dmenu -p "${title}")"
                    [[ -n "$profile" ]] || exit
                    ${pkgs.autorandr}/bin/autorandr ${option} "$profile"
                  '';
              in
              {
                "a" = "exec ${pkgs.autorandr}/bin/autorandr --change --force, mode default";
                "l" = "exec ${autorandrmenu {title="Load profile"; option="--load"; builtin = true;}}, mode default";
                "s" = "exec ${autorandrmenu {title="Save profile"; option="--save";}}, mode default";
                "r" = "exec ${autorandrmenu {title="Remove profile"; option="--remove";}}, mode default";
                "d" = "exec ${autorandrmenu {title="Default profile"; option="--default"; builtin = true;}}, mode default";
              } // return_bindings;
          };
      };
  };

  programs = {
    # Browser
    qutebrowser.enable = true;

    # Terminal
    alacritty = {
      # FIXME Emojis
      enable = true;
      settings = {
        bell = {
          animation = "EaseOutExpo";
          color = "#000000";
          command = { program = "${pkgs.sox}/bin/play"; args = [ "-n" "synth" "sine" "C5" "sine" "E4" "remix" "1-2" "fade" "0.1" "0.2" "0.1" ]; };
          duration = 100;
        };
        cursor = { vi_mode_style = "Underline"; };
        hints = {
          enabled = [
            {
              binding = { mods = "Control|Alt"; key = "F"; };
              command = "${pkgs.xdg-utils}/bin/xdg-open";
              mouse = { enabled = true; mods = "Control"; };
              post_processing = true;
              regex = "(mailto:|gemini:|gopher:|https:|http:|news:|file:|git:|ssh:|ftp:)[^\\u0000-\\u001F\\u007F-\\u009F<>\"\\\\s{-}\\\\^⟨⟩`]+";
            }
          ];
        };
        key_bindings = [
          { mode = "~Search"; mods = "Alt|Control"; key = "Space"; action = "ToggleViMode"; }
          { mode = "Vi|~Search"; mods = "Control"; key = "K"; action = "ScrollHalfPageUp"; }
          { mode = "Vi|~Search"; mods = "Control"; key = "J"; action = "ScrollHalfPageDown"; }
          { mode = "~Vi"; mods = "Control|Alt"; key = "V"; action = "Paste"; }
          { mods = "Control|Alt"; key = "C"; action = "Copy"; }
          { mode = "~Search"; mods = "Control|Alt"; key = "F"; action = "SearchForward"; }
          { mode = "~Search"; mods = "Control|Alt"; key = "B"; action = "SearchBackward"; }
          { mode = "Vi|~Search"; mods = "Control|Alt"; key = "C"; action = "ClearSelection"; }
        ];
        window = {
          dynamic_padding = false;
          dynamic_title = true;
        };
      };
    };
    # Backup terminal
    urxvt = {
      enable = true;
      package = pkgs.rxvt-unicode-emoji;
    };
    rofi = {
      enable = true;
      pass.enable = true;
    };
  };

  home.packages = with pkgs; [
    # remote
    tigervnc

    # music
    mpd
    mpc-cli
    ashuffle
    vimpc

    # multimedia common
    gimp
    inkscape
    mpv
    mpvScripts.thumbnail
    libreoffice

    # data management
    freefilesync

    # browsers
    firefox

    # fonts
    dejavu_fonts
    twemoji-color-font
    gnome.gedit
    feh
    zathura
    zbar
    zathura
    meld
    python3Packages.magic
    yubikey-touch-detector

    # x11-exclusive
    dunst
    numlockx
    # TODO urxvt-resize-font-git
    simplescreenrecorder
    trayer
    unclutter
    xautolock
    xclip
    lemonbar-xft
    keynav
    sct
    xorg.xinit
    xorg.xbacklight


    # organisation
    pass
    thunderbird
  ];

}
